@@ -168,7 +168,7 @@ MediaObject *MediaManager::createAVMedia (Node *node, const QByteArray &) {
 }
 
 static const QString statemap [] = {
-    i18n ("Not Running"), i18n ("Ready"), i18n ("Buffering"), i18n ("Playing")
+    i18n ("Not Running"), i18n ("Ready"), i18n ("Buffering"), i18n ("Playing"),  i18n ("Paused")
 };
 
 void MediaManager::stateChange (AudioVideoMedia *media,
@@ -193,11 +193,8 @@ void MediaManager::stateChange (AudioVideoMedia *media,
     m_player->updateStatus (i18n ("Player %1 %2",
                 media->process->process_info->name, statemap[news]));
     if (IProcess::Playing == news) {
-        if (Element::state_deferred == mrl->state) {
-            media->ignore_pause = true;
+        if (Element::state_deferred == mrl->state)
             mrl->undefer ();
-            media->ignore_pause = false;
-        }
         bool has_video = !is_rec;
         if (is_rec) {
             const ProcessList::iterator i = m_recorders.find (media->process);
@@ -240,7 +237,7 @@ void MediaManager::stateChange (AudioVideoMedia *media,
             }
             if (AudioVideoMedia::ask_delete == media->request) {
                 delete media;
-            } else if (olds > IProcess::Ready && mrl->unfinished ()) {
+            } else if (olds > IProcess::Ready) {
                 if (is_rec)
                     mrl->message (MsgMediaFinished, NULL); // FIXME
                 else
@@ -251,9 +248,7 @@ void MediaManager::stateChange (AudioVideoMedia *media,
         if (AudioVideoMedia::ask_pause == media->request) {
             media->pause ();
         } else if (mrl->view_mode != Mrl::SingleMode) {
-            media->ignore_pause = true;
             mrl->defer (); // paused the SMIL
-            media->ignore_pause = false;
         }
     }
 }
@@ -289,7 +284,7 @@ void MediaManager::processDestroyed (IProcess *process) {
 //------------------------%<----------------------------------------------------
 
 MediaObject::MediaObject (MediaManager *manager, Node *node)
- : m_manager (manager), m_node (node), paused (false) {
+ : m_manager (manager), m_node (node) {
     manager->medias ().push_back (this);
 }
 
@@ -434,7 +429,8 @@ KDE_NO_EXPORT bool MediaInfo::wget (const QString &str, const QString &domain) {
     Mrl *mrl = node->mrl ();
     if (mrl && (MediaManager::Any == type || MediaManager::AudioVideo == type))
     {
-        mime = mrl->mimetype;
+        if (!mrl->mimetype.isEmpty ())
+            setMimetype (mrl->mimetype);
         if (mrl && (MediaManager::Any == type || MediaManager::AudioVideo == type))
             if (mime == "application/x-shockwave-flash" ||
                     mime == "application/futuresplash" ||
@@ -459,7 +455,9 @@ KDE_NO_EXPORT bool MediaInfo::wget (const QString &str, const QString &domain) {
 
     bool only_playlist = false;
     bool maybe_playlist = false;
-    if (MediaManager::Audio == type || MediaManager::AudioVideo == type) {
+    if (MediaManager::Audio == type
+            || MediaManager::AudioVideo == type
+            || MediaManager::Any == type) {
         only_playlist = true;
         maybe_playlist = isPlayListMime (mime);
     }
@@ -472,12 +470,14 @@ KDE_NO_EXPORT bool MediaInfo::wget (const QString &str, const QString &domain) {
                 if (mrl && mimeptr) {
                     mrl->mimetype = mimeptr->name ();
                     setMimetype (mrl->mimetype);
-                    only_playlist = MediaManager::Audio == type ||
-                        MediaManager::AudioVideo == type;
-                    maybe_playlist = isPlayListMime (mime); // get new mime
                 }
                 kDebug () << "wget2 " << str << " " << mime;
+            } else {
+                setMimetype (mime);
             }
+            only_playlist = MediaManager::Audio == type ||
+                MediaManager::AudioVideo == type;
+            maybe_playlist = isPlayListMime (mime); // get new mime
             if (file.open (IO_ReadOnly)) {
                 if (only_playlist) {
                     maybe_playlist &= file.size () < 2000000;
@@ -515,9 +515,12 @@ KDE_NO_EXPORT bool MediaInfo::wget (const QString &str, const QString &domain) {
     if (!check_access) {
         if (MediaManager::Data != type &&
                 (memory_cache->get (str, mime, data) ||
-                 protocol == "mms" || protocol == "rtsp" || protocol == "rtp" ||
+                 protocol == "mms" || protocol == "rtsp" ||
+                 protocol == "rtp" || protocol == "rtmp" ||
                  (only_playlist && !maybe_playlist && !mime.isEmpty ()))) {
             setMimetype (mime);
+            if (MediaManager::Any == type)
+                type = MediaManager::AudioVideo;
             ready ();
             return true;
         }
@@ -573,7 +576,7 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
                     } else if (pls_groupfound) {
                         int eq_pos = line.indexOf (QChar ('='));
                         if (eq_pos > 0) {
-                            if (line.lower ().startsWith (QString ("numberofentries"))) {
+                            if (line.toLower ().startsWith (QString ("numberofentries"))) {
                                 nr = line.mid (eq_pos + 1).stripWhiteSpace ().toInt ();
                                 kDebug () << "numberofentries : " << nr;
                                 if (nr > 0 && nr < 1024)
@@ -581,7 +584,7 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
                                 else
                                     nr = 0;
                             } else if (nr > 0) {
-                                QString ll = line.lower ();
+                                QString ll = line.toLower ();
                                 if (ll.startsWith (QString ("file"))) {
                                     int i = line.mid (4, eq_pos-4).toInt ();
                                     if (i > 0 && i <= nr)
@@ -607,7 +610,7 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
         } else if (line.stripWhiteSpace ().startsWith (QChar ('<'))) {
             readXML (cur_elm, textstream, line);
             //cur_elm->normalize ();
-        } else if (line.lower () != QString ("[reference]")) {
+        } else if (line.toLower () != QString ("[reference]")) {
             bool extm3u = line.startsWith ("#EXTM3U");
             QString title;
             NodePtr doc = node->document ();
@@ -618,7 +621,7 @@ KDE_NO_EXPORT bool MediaInfo::readChildDoc () {
                 QString mrl = line.stripWhiteSpace ();
                 if (line == QString ("--stop--"))
                     break;
-                if (mrl.lower ().startsWith (QString ("asf ")))
+                if (mrl.toLower ().startsWith (QString ("asf ")))
                     mrl = mrl.mid (4).stripWhiteSpace ();
                 if (!mrl.isEmpty ()) {
                     if (extm3u && mrl.startsWith (QChar ('#'))) {
@@ -753,7 +756,7 @@ KDE_NO_EXPORT void MediaInfo::slotResult (KJob *kjob) {
                     "//cross-domain-policy/allow-access-from/@domain");
             if (expr) {
                 expr->setRoot (doc);
-                NodeValueList *lst = expr->toNodeList ();
+                Sequence *lst = expr->toSequence ();
                 for (NodeValueItem *i = lst->first(); i; i = i->nextSibling()) {
                     QRegExp match (i->data.value (), Qt::CaseInsensitive, QRegExp::Wildcard);
                     if (match.exactMatch (access_from)) {
@@ -846,8 +849,7 @@ AudioVideoMedia::AudioVideoMedia (MediaManager *manager, Node *node)
  : MediaObject (manager, node),
    process (NULL),
    m_viewer (NULL),
-   request (ask_nothing),
-   ignore_pause (false) {
+   request (ask_nothing) {
     kDebug() << "AudioVideoMedia::AudioVideoMedia" << endl;
 }
 
@@ -909,9 +911,8 @@ void AudioVideoMedia::stop () {
 }
 
 void AudioVideoMedia::pause () {
-    if (!ignore_pause && !paused && process) {
+    if (process) {
         if (process->state () > IProcess::Ready) {
-            paused = true;
             request = ask_nothing;
             process->pause ();
         } else {
@@ -921,12 +922,11 @@ void AudioVideoMedia::pause () {
 }
 
 void AudioVideoMedia::unpause () {
-    if (!ignore_pause && paused && process) {
+    if (process) {
         if (request == ask_pause) {
             request = ask_nothing;
         } else {
-            paused = false;
-            process->pause ();
+            process->unpause ();
         }
     }
 }
